#! /usr/bin/env python

import cmdsyntax, sys

# Define the syntax: note that the CMDSyntax library doesn't support the
# usage of the -v option which allows -vv and -vvv, etc.

syntax = """
  [-h | --help] | [--version] |
  (
    [
      (-v | --verbose=V) | (-q | --quiet)
    ]
    
    [( -d DEVICE) | (--device=DEVICE)]
    [(-b DIR) | --output-base=DIR]
  
    [(-t TRACKS) | --tracks=TRACKS]
  
    [
      (-p | --use-pipes | --synchronous)
      |
      (
        (-f | --use-files | --asynchronous)
        [
            (-k | --keep-tmp) | --no-keep-tmp
            [-R | --rip-only]
        ]
      )
    ]
    
    [-P | --playback]
    [-e | --eject]
    [(-A ARTIST) | --artist=ARTIST]
    [(-L ALBUM) | --album=ALBUM]
    [(-D DISC) | --disc=DISC]
    [(-y YEAR) | --year=YEAR]
    [--offset=OFFSET]
  )
"""

if __name__ == "__main__":

    # Create a syntax object.
    syntax_obj = cmdsyntax.Syntax(syntax)
    
    # Match the command line arguments against the syntax definition.
    matches = syntax_obj.get_args(sys.argv[1:])
    
    if matches == [] and cmdsyntax.use_GUI() != None:
        
        print "Using the GUI..."
        print
        
        form = cmdsyntax.Form(sys.argv[0], syntax_obj)
        
        match = form.get_args()
    
        # A valid match was found.
        if match:
        
            # Put the unambiguous match in a list so that it can be dealt
            # with in the same manner as matches from command line input.
            matches = [match]
    
    if matches == []:
    
        # No matches found: print the syntax definition and exit.
        
        print "Syntax: %s %s" % (sys.argv[0], syntax)
        sys.exit()
    
    # At this point, there may be a number of matches if the syntax
    # specification was ambiguous. Assuming that it wasn't, take the
    # first match.
    
    match = matches[0]
    
    # Note that the CMDSyntax library does not coerce command line arguments
    # to types; this is up to the application. Using the source for the
    # Optik solution to ripoff, we can determine which arguments have
    # types other than the string type.
    
    def coerce_using(value, fn):
    
        try:
            return fn(value)
        
        except ValueError:
        
            print "Could not coerce %s using %s." % (value, fn)
            return value
    
    # Integer types:
    
    if match.has_key("verbose"):
    
        match["verbose"] = coerce_using(match["verbose"], int)
    
    if match.has_key("disc"):
    
        match["disc"] = coerce_using(match["disc"], int)
    
    if match.has_key("DISC"):
    
        match["DISC"] = coerce_using(match["DISC"], int)
    
    if match.has_key("offset"):
    
        match["offset"] = coerce_using(match["offset"], int)
    
    print match
    
    sys.exit()
