#! /usr/bin/env python

import sys, cmdsyntax, string, types

# Add a syntax definition here.

# Syntax which won't work:
#syntax = "infile -a=badger [-b=hippo] -c (giraffe) -d|-e"

# Syntax which should work:
#syntax = "--dir=directory"
#syntax = "[--dir=directory]"
#syntax = "infile (-a|-b|-c) (cat|dog|hippo)"
#syntax = 'infile "add"|"remove"|"catalogue"'
#syntax = "infile [ -a | -b ]"
#syntax = "infile -abc"
#syntax = "[-a] [-b]"
#syntax = "infile [ -abc ]"
#syntax = "infile"
#syntax = "(-d dir) | --directory=dir infile"
#syntax = "infile (-d dir) | --directory=dir"
#syntax = "[(-d dir) | --directory=dir] infile"
#syntax = "[(-d dir) | --output-dir=dir] infile"
#syntax = " <input file> [-o <output file>]"
#syntax = "[-a] [[-b badger] -c]"
#syntax = "[-a] [-b [-c]]"
#syntax = '("group"|"g" name [-p pages])|("read"|"r" number)'
#syntax = """(
#                "group" | "g"
#                name
#                [
#                    -p
#                    pages
#                ]
#            ) |
#            (
#                "read"    |    "r"
#                number
#            )"""

# A good test for the optimisation of selections.
#syntax = "-h | [-b]"
#syntax = (
#    "--help | ("
#    "[-b <first frame>] [-e <last frame>] [<bounding box>] [<image size>] "
#    "<input file> <output directory> )"
#    )

# Convenient general non-GNU style syntax.
#syntax = "infile [-a abacus] [-b binary] outfile [-c computer]"

#syntax = "[(-a abacus)] [[-h hippo]]"
#syntax = "[((-a))] [(-b)]"

# Syntax which will generated matches that will be nonsensical.
#syntax = "infile -a infile"

# Simple, multi-line syntax:
#syntax = \
#"""
#-a -b 
#-c
#"""

# Syntax involving selections:

# Ambiguous selection:
#syntax = "{henry bill jeremy}"

# Unambiguous selection:
#syntax = "{(-h hostname) (-a address)}"
#syntax = "{(--hostname=name) (--address=addr)}"
#syntax = "{--send-e-mail --send-postcard --telephone-call}"
#syntax = """{ --send-e-mail=emailaddress --send-postcard=postaladdress
#              --telephone-call=phonenumber }"""

# Selections with minimum numbers of required arguments

# Options in any order:
#syntax = "{#3 --hippopotamus --giraffe --lion}"

#syntax = "{#0 --name=s --red=n --green=n --blue=n}"
#syntax = "{#1 --name=s --red=n --green=n --blue=n}"
#syntax = "{#2 --name=s --red=n --green=n --blue=n}"
#syntax = "{#3 --name=s --red=n --green=n --blue=n}"
#syntax = "{#4 --name=s --red=n --green=n --blue=n}"

# Complicated example:
#syntax = \
#"""
#    {   --named-colour=s
#        {#3 --red=i --green=i --blue=i}
#        {#4 --cyan=i --magenta=i --yellow=i --black=i}
#    }
#"""

#syntax = "{#2-2 --hold-sword --hold-shield --hold-axe --hold-longbow}"

# Multiple item syntaxes:
#syntax = "infile ..."
#syntax = "infile ... -a"

# Multiple item syntax which can be matched but will be nonsensical.
#syntax = "infile ... -a infile"

# Ambiguous multiple item syntax:
#syntax = "infile ... [-a]"

# A syntax which allows options to be specified more than once:
#syntax = "-abc ..."

# A syntax which allows two exclusive options to be specified together:
#syntax = '("hello"|"goodbye") ...'

#syntax = "input_file ... [-o output_file ...]"

#syntax = "{--name=s --red=n --green=n --blue=n} ..."

#syntax = "[name --address=addr] ..."

# Selections with minimum numbers of required arguments

#syntax = "{#0 --name=s --red=n --green=n --blue=n} ..."
#syntax = "{#1 --name=s --red=n --green=n --blue=n} ..."
#syntax = "{#2 --name=s --red=n --green=n --blue=n} ..."
#syntax = "{#3 --name=s --red=n --green=n --blue=n} ..."
#syntax = "{#4 --name=s --red=n --green=n --blue=n} ..."

# Real world syntaxes
#syntax = "<Input file> <Output directory> <File extension>" + \
#         " [-s <Starting page>] [-f <Finishing page>]"

#syntax = """
#    (
#        (-l | --list) [-t | --file-types] <ADF file>
#    ) |
#    (
#        [-d | --create-directory]
#        [(-t | --file-types) [(-s separator) | --separator=character]]
#        <ADF file> <destination path>
#    ) |
#    (
#        (-v | --verify) <ADF file>
#    )
#    """
#
#syntax = "<Input file> <Output directory> <File extension>" + \
#         " [-s <Starting page>] [-f <Finishing page>]"
#
#syntax = \
#    """( "build" <path> [patterns]) |
#    \r( "changed" <path> <archive> [--allow-unknown] ) |
#    \r( "archive" <path> <archive> [--show-changes | --write-changes=file]
#    \r  [--commit --comment=text] )
#    """
#
#syntax = "--help | " + \
#    "(" + \
#        "--suffixes suffix ... --paths path ... " + \
#        "--show-dirs | --show-files | --show-all" + \
#    ")"

# Badly specified syntax:
#syntax = "infile [-a] [-b binary] outfile [-c] logfile [status] [addr] [port]"


def read_list(l, indent):

    ind = '    '*indent
    
    count = 0
    
    if isinstance(l, cmdsyntax.Selection):
    
        op = 'OR'
        l = l.objects
        
    else:
    
        op = 'AND'
        
    for i in l:
    
        print ind + "%i" % count,
        
        if type(i) == types.StringType:
        
            print '    label: ' + i
        
        elif isinstance(i, cmdsyntax.Command):
        
            print '    command: ' + i.name
        
        elif isinstance(i, cmdsyntax.Option):
        
            if i.value == None:
                print '    option: ' + i.name
            else:
                print '    option: ' + i.name + " = " + i.value
            
        elif isinstance(i, cmdsyntax.Selection):
        
            if i.minimum != i.maximum:
            
                if i.minimum > 0:
                    print "    select %i to %i from" % (i.minimum, i.maximum)
                else:
                    print "    select up to %i from" % i.maximum
            else:
                print "    select %i from" % i.minimum
            print
            read_list(i, indent+1)
        
        elif isinstance(i, cmdsyntax.Multiple):
        
            print "    One or more of"
            print
            read_list([i.item], indent+1)
        
        elif type(i) == types.ListType:
        
            print
            read_list(i, indent+1)
        
        else:
        
            print "Unknown object found."
            return

        if i is not l[-1]:
            print ind + ' '+op

        count = count + 1
    
    return


def show_match(match):

    if match == {}:
    
        print "Empty dictionary"
        return
    
    key_length = 0
    for key in match.keys():
    
        key_length = max( len(key), key_length )

    key_length = ((key_length / 8) + ((key_length % 8) != 0) ) * 8
    for key in match.keys():
    
        print string.expandtabs(key+"\t:\t"+str(match[key]), key_length)


if __name__ == "__main__":

    try:
    
        print "Syntax: "+sys.argv[0]+" "+syntax
    
    except NameError:
    
        print "Add a syntax definition to the test script then run it again."
        sys.exit()

    syntax_obj = cmdsyntax.Syntax(syntax)

    print 
    print "Syntax tree:"
    print
    read_list(syntax_obj.syntax_tree, 2)
    print
    #print "Cleaned syntax:"
    #print
    #print syntax_obj.pprint()
    #print

    matches, failed = syntax_obj.get_args(sys.argv[1:], return_failed = 1)
    
    if matches == []:
    
        print "No matches found."
        print
        print "Failed matches:"
        print
        
        for match in failed:
        
            show_match(match)
            print

    else:
    
        print "Matches:"
        print
        
        for match in matches:
        
            show_match(match)
            print
        
        if len(failed) > 0:
        
            print "%i failed matches were also found:" % len(failed)
            print
            
            for match in failed:
            
                show_match(match)
                print
    
    if matches == [] and cmdsyntax.use_GUI("PyQt") != None:
#    if matches == [] and cmdsyntax.use_GUI() != None:
        
        print "Using the GUI..."
        print
        
        # Choose the most extensive failed match.
        closest = failed[0]
        
        form = cmdsyntax.Form(sys.argv[0], syntax_obj, closest)
        
        matches = form.get_args()
        
        if matches != []:
        
            match = matches[0]
            
            print "Match:"
            print
            
            show_match(match)
            print
    
        else:
        
            print "No match found."
            print
    
    sys.exit()
